._uploadDragger_1e6jb_2{width:600px;height:280px}._uploadDragger_1e6jb_2 .ant-upload-drag{background:#fff;border:Mixed solid rgba(215,216,221,1)}._fileList_1e6jb_12{height:calc(100vh - 88px);border-top-left-radius:0;border-bottom-left-radius:0}._fileList_1e6jb_12 .ant-list-item-meta-title{overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:180px}._fileList_1e6jb_12>.ant-spin-nested-loading{flex:1;min-height:0;overflow:auto}
